@@ -45,23 +45,23 @@
 #include <string.h>
 #include <stdio.h>
 
-#if HAVE_SYS_SOCKET_H
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
 #endif
 #include <sys/types.h>
-#if HAVE_NETINET_IN_H
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
-#include <unistd.h>
-#if HAVE_NETINET_IP_H
+#ifdef HAVE_NETINET_IP_H
 #include <netinet/ip.h>
 #endif
-#if HAVE_NETINET_TCP_H
+#ifdef HAVE_NETINET_TCP_H
 #include <netinet/tcp.h>
 #endif
 #include <sys/stat.h>
 #include <fcntl.h>
 
+#include "gstcurlelements.h"
 #include "gstcurltlssink.h"
 #include "gstcurlftpsink.h"
 
@@ -101,6 +101,8 @@ static gboolean set_ftp_dynamic_options_unlocked
 
 #define gst_curl_ftp_sink_parent_class parent_class
 G_DEFINE_TYPE (GstCurlFtpSink, gst_curl_ftp_sink, GST_TYPE_CURL_TLS_SINK);
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (curlftpsink, "curlftpsink",
+    GST_RANK_NONE, GST_TYPE_CURL_FTP_SINK, curl_element_init (plugin));
 
 static void
 gst_curl_ftp_sink_class_init (GstCurlFtpSinkClass * klass)
