@@ -46,6 +46,7 @@
 #if GST_VULKAN_HAVE_VIDEO_EXTENSIONS
 #include "vkh264dec.h"
 #include "vkh265dec.h"
+# include "vkh264enc.h"
 #endif
 
 static gboolean
@@ -53,7 +54,8 @@ plugin_init (GstPlugin * plugin)
 {
   gboolean ret = FALSE;
   GstVulkanInstance *instance = gst_vulkan_instance_new ();
-  gboolean have_instance = gst_vulkan_instance_open (instance, NULL);
+  GError *error = NULL;
+  gboolean have_instance = gst_vulkan_instance_open (instance, &error);
   const gchar *env_vars[] =
       { "VK_ICD_FILENAMES", "VK_DRIVER_FILES", "VK_ADD_DRIVER_FILES", NULL };
 #ifndef G_OS_WIN32
@@ -69,6 +71,12 @@ plugin_init (GstPlugin * plugin)
   gst_plugin_add_dependency (plugin, env_vars, NULL, NULL,
       GST_PLUGIN_DEPENDENCY_FLAG_NONE);
 
+  if (!have_instance) {
+    GST_WARNING_OBJECT (plugin, "Failed to create vulkan instance: %s",
+        error->message);
+    g_clear_error (&error);
+  }
+
   ret |= GST_DEVICE_PROVIDER_REGISTER (vulkandeviceprovider, plugin);
 
   ret |= GST_ELEMENT_REGISTER (vulkanupload, plugin);
@@ -97,6 +105,10 @@ plugin_init (GstPlugin * plugin)
               VK_KHR_VIDEO_DECODE_H265_EXTENSION_NAME)) {
         ret |= gst_vulkan_h265_decoder_register (plugin, device, GST_RANK_NONE);
       }
+      if (gst_vulkan_device_is_extension_enabled (device,
+              VK_KHR_VIDEO_ENCODE_H264_EXTENSION_NAME)) {
+        ret |= gst_vulkan_h264_encoder_register (plugin, device, GST_RANK_NONE);
+      }
 #endif
       ret |= gst_vulkan_sink_register (plugin, device, GST_RANK_NONE);
       gst_object_unref (device);
