@@ -51,7 +51,7 @@ _set_caps_features_with_passthrough (const GstCaps * caps,
     GstCapsFeatures *features, *orig_features;
 
     orig_features = gst_caps_get_features (caps, i);
-    features = gst_caps_features_new (feature_name, NULL);
+    features = gst_caps_features_new_static_str (feature_name, NULL);
 
     m = gst_caps_features_get_size (orig_features);
     for (j = 0; j < m; j++) {
@@ -197,8 +197,9 @@ _image_to_raw_perform (gpointer impl, GstBuffer * inbuf, GstBuffer ** outbuf)
   cmd_buf = raw->exec->cmd_buf;
 
   if (!gst_vulkan_operation_add_frame_barrier (raw->exec, inbuf,
-          VK_PIPELINE_STAGE_TRANSFER_BIT, VK_ACCESS_TRANSFER_READ_BIT,
-          VK_IMAGE_LAYOUT_TRANSFER_SRC_OPTIMAL, NULL))
+          VK_PIPELINE_STAGE_ALL_COMMANDS_BIT, VK_PIPELINE_STAGE_TRANSFER_BIT,
+          VK_ACCESS_TRANSFER_READ_BIT, VK_IMAGE_LAYOUT_TRANSFER_SRC_OPTIMAL,
+          NULL))
     goto unlock_error;
 
   barriers = gst_vulkan_operation_retrieve_image_barriers (raw->exec);
@@ -440,7 +441,7 @@ gst_vulkan_download_class_init (GstVulkanDownloadClass * klass)
   gstelement_class = (GstElementClass *) klass;
   gstbasetransform_class = (GstBaseTransformClass *) klass;
 
-  gst_element_class_set_metadata (gstelement_class, "Vulkan Downloader",
+  gst_element_class_set_static_metadata (gstelement_class, "Vulkan Downloader",
       "Filter/Video", "A Vulkan data downloader",
       "Matthew Waters <matthew@centricular.com>");
 
@@ -568,32 +569,34 @@ gst_vulkan_download_change_state (GstElement * element,
             ("Failed to retrieve vulkan instance"), (NULL));
         return GST_STATE_CHANGE_FAILURE;
       }
-      if (!gst_vulkan_device_run_context_query (GST_ELEMENT (vk_download),
-              &vk_download->device)) {
-        GError *error = NULL;
-        GST_DEBUG_OBJECT (vk_download,
-            "No device retrieved from peer elements");
-        if (!(vk_download->device =
-                gst_vulkan_instance_create_device (vk_download->instance,
-                    &error))) {
-          GST_ELEMENT_ERROR (vk_download, RESOURCE, NOT_FOUND,
-              ("Failed to create vulkan device"), ("%s",
-                  error ? error->message : ""));
-          g_clear_error (&error);
-          return GST_STATE_CHANGE_FAILURE;
+      if (!gst_vulkan_ensure_element_device (element, vk_download->instance,
+              &vk_download->device, 0)) {
+        return GST_STATE_CHANGE_FAILURE;
+      }
+
+      if (gst_vulkan_queue_run_context_query (GST_ELEMENT (vk_download),
+              &vk_download->queue)) {
+        guint32 flags, idx;
+
+        GST_DEBUG_OBJECT (vk_download, "Queue retrieved from peer elements");
+        idx = vk_download->queue->family;
+        flags = vk_download->device->physical_device->queue_family_props[idx]
+            .queueFlags;
+        if ((flags & (VK_QUEUE_GRAPHICS_BIT | VK_QUEUE_TRANSFER_BIT)) == 0) {
+          GST_DEBUG_OBJECT (vk_download,
+              "Queue does not support VK_QUEUE_GRAPHICS_BIT with VK_QUEUE_TRANSFER_BIT");
+          gst_clear_object (&vk_download->queue);
         }
       }
 
-      if (!gst_vulkan_queue_run_context_query (GST_ELEMENT (vk_download),
-              &vk_download->queue)) {
-        GST_DEBUG_OBJECT (vk_download, "No queue retrieved from peer elements");
+      if (!vk_download->queue) {
         vk_download->queue =
             gst_vulkan_device_select_queue (vk_download->device,
             VK_QUEUE_GRAPHICS_BIT);
       }
       if (!vk_download->queue) {
         GST_ELEMENT_ERROR (vk_download, RESOURCE, NOT_FOUND,
-            ("Failed to create/retrieve vulkan queue"), (NULL));
+            ("Failed to create/retrieve a valid vulkan queue"), (NULL));
         return GST_STATE_CHANGE_FAILURE;
       }
       break;
