@@ -239,13 +239,33 @@ _raw_to_buffer_propose_allocation (gpointer impl, GstQuery * decide_query,
   _buffer_propose_allocation (impl, decide_query, query);
 }
 
+static gboolean
+_copy_frames (const GstVideoInfo * vinfo, GstBuffer * inbuf, GstBuffer * outbuf)
+{
+  GstVideoFrame in_frame, out_frame;
+  gboolean copied;
+
+  if (!gst_video_frame_map (&in_frame, vinfo, inbuf, GST_MAP_READ))
+    return FALSE;
+
+  if (!gst_video_frame_map (&out_frame, vinfo, outbuf, GST_MAP_WRITE)) {
+    gst_video_frame_unmap (&in_frame);
+    return FALSE;
+  }
+
+  copied = gst_video_frame_copy (&out_frame, &in_frame);
+
+  gst_video_frame_unmap (&in_frame);
+  gst_video_frame_unmap (&out_frame);
+
+  return copied;
+}
+
 static GstFlowReturn
 _raw_to_buffer_perform (gpointer impl, GstBuffer * inbuf, GstBuffer ** outbuf)
 {
   struct RawToBufferUpload *raw = impl;
-  GstVideoFrame v_frame;
-  GstFlowReturn ret;
-  guint i, n_mems;
+  GstFlowReturn ret = GST_FLOW_ERROR;
   GstBufferPool *pool;
 
   pool = gst_base_transform_get_buffer_pool
@@ -258,41 +278,12 @@ _raw_to_buffer_perform (gpointer impl, GstBuffer * inbuf, GstBuffer ** outbuf)
       != GST_FLOW_OK)
     goto out;
 
-  if (!gst_video_frame_map (&v_frame, &raw->in_info, inbuf, GST_MAP_READ)) {
+  ret = _copy_frames (&raw->in_info, inbuf, *outbuf);
+  if (!ret) {
     GST_ELEMENT_ERROR (raw->upload, RESOURCE, NOT_FOUND,
         ("%s", "Failed to map input buffer"), NULL);
-    return GST_FLOW_ERROR;
   }
 
-  n_mems = gst_buffer_n_memory (*outbuf);
-  for (i = 0; i < n_mems; i++) {
-    GstMapInfo map_info;
-    gsize plane_size;
-    GstMemory *mem;
-
-    mem = gst_buffer_peek_memory (*outbuf, i);
-    if (!gst_memory_map (GST_MEMORY_CAST (mem), &map_info, GST_MAP_WRITE)) {
-      GST_ELEMENT_ERROR (raw->upload, RESOURCE, NOT_FOUND,
-          ("%s", "Failed to map output memory"), NULL);
-      gst_buffer_unref (*outbuf);
-      *outbuf = NULL;
-      ret = GST_FLOW_ERROR;
-      goto out;
-    }
-
-    plane_size =
-        GST_VIDEO_INFO_PLANE_STRIDE (&raw->out_info,
-        i) * GST_VIDEO_INFO_COMP_HEIGHT (&raw->out_info, i);
-    g_assert (plane_size < map_info.size);
-    memcpy (map_info.data, v_frame.data[i], plane_size);
-
-    gst_memory_unmap (GST_MEMORY_CAST (mem), &map_info);
-  }
-
-  gst_video_frame_unmap (&v_frame);
-
-  ret = GST_FLOW_OK;
-
 out:
   gst_object_unref (pool);
   return ret;
@@ -392,6 +383,7 @@ _buffer_to_image_perform (gpointer impl, GstBuffer * inbuf, GstBuffer ** outbuf)
   GArray *barriers = NULL;
   VkImageLayout dst_layout = VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL;
   GstBufferPool *pool;
+  GstVideoMeta *in_vmeta, *out_vmeta;
 
   pool = gst_base_transform_get_buffer_pool
       (GST_BASE_TRANSFORM_CAST (raw->upload));
@@ -458,8 +450,11 @@ _buffer_to_image_perform (gpointer impl, GstBuffer * inbuf, GstBuffer ** outbuf)
   g_clear_pointer (&barriers, g_array_unref);
 
   n_mems = gst_buffer_n_memory (*outbuf);
+  out_vmeta = gst_buffer_get_video_meta (*outbuf);
   n_planes = GST_VIDEO_INFO_N_PLANES (&raw->out_info);
 
+  in_vmeta = gst_buffer_get_video_meta (inbuf);
+
   for (i = 0; i < n_planes; i++) {
     VkBufferImageCopy region;
     GstMemory *in_mem, *out_mem;
@@ -469,9 +464,19 @@ _buffer_to_image_perform (gpointer impl, GstBuffer * inbuf, GstBuffer ** outbuf)
       VK_IMAGE_ASPECT_PLANE_1_BIT, VK_IMAGE_ASPECT_PLANE_2_BIT,
     };
     VkImageAspectFlags plane_aspect;
-    guint idx;
+    guint idx, len;
+    gsize offset, skip;
 
+    offset = in_vmeta ? in_vmeta->offset[i]
+        : GST_VIDEO_INFO_PLANE_OFFSET (&raw->in_info, i);
+    if (!gst_buffer_find_memory (inbuf, offset, 1, &idx, &len, &skip)) {
+      GST_WARNING_OBJECT (raw->upload,
+          "Input buffer plane %u, no memory at offset %" G_GSIZE_FORMAT, i,
+          offset);
+      goto unlock_error;
+    }
     in_mem = gst_buffer_peek_memory (inbuf, i);
+
     if (!gst_is_vulkan_buffer_memory (in_mem)) {
       GST_WARNING_OBJECT (raw->upload, "Input is not a GstVulkanBufferMemory");
       goto unlock_error;
@@ -502,8 +507,16 @@ _buffer_to_image_perform (gpointer impl, GstBuffer * inbuf, GstBuffer ** outbuf)
         }
     };
 
-    idx = MIN (i, n_mems - 1);
+    offset = out_vmeta ? out_vmeta->offset[i]
+        : GST_VIDEO_INFO_PLANE_OFFSET (&raw->out_info, i);
+    if (!gst_buffer_find_memory (*outbuf, offset, 1, &idx, &len, &skip)) {
+      GST_WARNING_OBJECT (raw->upload,
+          "Output buffer plane %u, no memory at offset %" G_GSIZE_FORMAT, i,
+          offset);
+      goto unlock_error;
+    }
     out_mem = gst_buffer_peek_memory (*outbuf, idx);
+
     if (!gst_is_vulkan_image_memory (out_mem)) {
       GST_WARNING_OBJECT (raw->upload, "Output is not a GstVulkanImageMemory");
       goto unlock_error;
@@ -654,9 +667,10 @@ _raw_to_image_perform (gpointer impl, GstBuffer * inbuf, GstBuffer ** outbuf)
   GstVulkanCommandBuffer *cmd_buf;
   GError *error = NULL;
   GArray *barriers = NULL;
-  guint i, n_mems, n_planes, n_out_mems;
+  guint i, n_planes, n_out_mems;
   VkImageLayout dst_layout = VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL;
   GstBufferPool *pool;
+  GstVideoMeta *in_vmeta, *out_vmeta;
 
   pool = gst_base_transform_get_buffer_pool
       (GST_BASE_TRANSFORM_CAST (raw->upload));
@@ -720,24 +734,33 @@ _raw_to_image_perform (gpointer impl, GstBuffer * inbuf, GstBuffer ** outbuf)
   }
   g_clear_pointer (&barriers, g_array_unref);
 
-  n_mems = gst_buffer_n_memory (inbuf);
+  in_vmeta = gst_buffer_get_video_meta (inbuf);
   n_out_mems = gst_buffer_n_memory (*outbuf);
-  if (n_mems != n_out_mems)
-    goto unlock_error;
+  out_vmeta = gst_buffer_get_video_meta (*outbuf);
   n_planes = GST_VIDEO_INFO_N_PLANES (&raw->in_info);
 
-  for (i = 0; i < n_mems; i++) {
+  for (i = 0; i < n_planes; i++) {
     VkBufferImageCopy region;
-    GstMemory *in_mem, *out_mem;
+    GstMemory *in_mem = NULL, *out_mem;
     GstVulkanBufferMemory *buf_mem;
     GstVulkanImageMemory *img_mem;
     const VkImageAspectFlags aspects[] = { VK_IMAGE_ASPECT_PLANE_0_BIT,
       VK_IMAGE_ASPECT_PLANE_1_BIT, VK_IMAGE_ASPECT_PLANE_2_BIT,
     };
     VkImageAspectFlags plane_aspect;
-    guint idx;
+    guint idx, len;
+    gsize offset, skip;
+
+    offset = in_vmeta ? in_vmeta->offset[i] :
+        GST_VIDEO_INFO_PLANE_OFFSET (&raw->in_info, i);
+    if (!gst_buffer_find_memory (inbuf, offset, 1, &idx, &len, &skip)) {
+      GST_WARNING_OBJECT (raw->upload,
+          "Input buffer plane %u, no memory at offset %" G_GSIZE_FORMAT, i,
+          offset);
+      goto unlock_error;
+    }
+    in_mem = gst_buffer_peek_memory (inbuf, idx);
 
-    in_mem = gst_buffer_peek_memory (inbuf, i);
     if (gst_is_vulkan_buffer_memory (in_mem)) {
       GST_TRACE_OBJECT (raw->upload, "Input is a GstVulkanBufferMemory");
       buf_mem = (GstVulkanBufferMemory *) in_mem;
@@ -748,8 +771,6 @@ _raw_to_image_perform (gpointer impl, GstBuffer * inbuf, GstBuffer ** outbuf)
       g_assert (gst_is_vulkan_buffer_memory (in_mem));
       buf_mem = (GstVulkanBufferMemory *) in_mem;
     } else {
-      GstVideoFrame in_frame, out_frame;
-
       GST_TRACE_OBJECT (raw->upload,
           "Copying input to a new GstVulkanBufferMemory");
       if (!raw->in_pool) {
@@ -774,41 +795,31 @@ _raw_to_image_perform (gpointer impl, GstBuffer * inbuf, GstBuffer ** outbuf)
         goto unlock_error;
       }
 
-      if (!gst_video_frame_map (&in_frame, &raw->in_info, inbuf, GST_MAP_READ)) {
-        GST_WARNING_OBJECT (raw->upload, "Failed to map input buffer");
+      if (!_copy_frames (&raw->in_info, inbuf, *outbuf)) {
+        GST_ERROR_OBJECT (raw->upload, "Failed to copy to Vulkan buffer");
         goto unlock_error;
       }
 
-      if (!gst_video_frame_map (&out_frame, &raw->in_info, in_vk_copy,
-              GST_MAP_WRITE)) {
-        gst_video_frame_unmap (&in_frame);
-        GST_WARNING_OBJECT (raw->upload, "Failed to map input buffer");
-        goto unlock_error;
-      }
-
-      if (!gst_video_frame_copy (&out_frame, &in_frame)) {
-        gst_video_frame_unmap (&in_frame);
-        gst_video_frame_unmap (&out_frame);
-        GST_WARNING_OBJECT (raw->upload, "Failed to copy input buffer");
-        goto unlock_error;
-      }
-
-      gst_video_frame_unmap (&in_frame);
-      gst_video_frame_unmap (&out_frame);
-
       in_mem = gst_buffer_peek_memory (in_vk_copy, i);
       buf_mem = (GstVulkanBufferMemory *) in_mem;
     }
 
-    idx = MIN (i, n_mems - 1);
+    offset = out_vmeta ? out_vmeta->offset[i] : GST_VIDEO_INFO_PLANE_OFFSET (&raw->out_info, i);
+    if (!gst_buffer_find_memory (*outbuf, offset, 1, &idx, &len, &skip)) {
+      GST_WARNING_OBJECT (raw->upload,
+          "Output buffer plane %u, no memory at offset %" G_GSIZE_FORMAT, i,
+          offset);
+      goto unlock_error;
+    }
     out_mem = gst_buffer_peek_memory (*outbuf, idx);
+
     if (!gst_is_vulkan_image_memory (out_mem)) {
       GST_WARNING_OBJECT (raw->upload, "Output is not a GstVulkanImageMemory");
       goto unlock_error;
     }
     img_mem = (GstVulkanImageMemory *) out_mem;
 
-    if (n_planes == n_mems)
+    if (n_planes == n_out_mems)
       plane_aspect = VK_IMAGE_ASPECT_COLOR_BIT;
     else
       plane_aspect = aspects[i];
@@ -1007,7 +1018,7 @@ gst_vulkan_upload_class_init (GstVulkanUploadClass * klass)
   gobject_class->set_property = gst_vulkan_upload_set_property;
   gobject_class->get_property = gst_vulkan_upload_get_property;
 
-  gst_element_class_set_metadata (gstelement_class, "Vulkan Uploader",
+  gst_element_class_set_static_metadata (gstelement_class, "Vulkan Uploader",
       "Filter/Video", "A Vulkan data uploader",
       "Matthew Waters <matthew@centricular.com>");
 
@@ -1336,6 +1347,20 @@ gst_vulkan_upload_propose_allocation (GstBaseTransform * bt,
   return TRUE;
 }
 
+static GstBufferPool *
+gst_vulkan_upload_create_buffer_pool (GstVulkanUpload * vk_upload)
+{
+  gboolean is_image;
+  GstCapsFeatures *feats = gst_caps_get_features (vk_upload->out_caps, 0);
+
+  is_image =
+      gst_caps_features_contains (feats, GST_CAPS_FEATURE_MEMORY_VULKAN_IMAGE);
+
+  if (is_image)
+    return gst_vulkan_image_buffer_pool_new (vk_upload->device);
+  return gst_vulkan_buffer_pool_new (vk_upload->device);
+}
+
 static gboolean
 gst_vulkan_upload_decide_allocation (GstBaseTransform * bt, GstQuery * query)
 {
@@ -1360,13 +1385,15 @@ gst_vulkan_upload_decide_allocation (GstBaseTransform * bt, GstQuery * query)
     gst_query_parse_nth_allocation_pool (query, 0, &pool, &size, &min, &max);
 
     update_pool = TRUE;
-    is_vulkan_pool = GST_IS_VULKAN_IMAGE_BUFFER_POOL (pool);
+    is_vulkan_pool = GST_IS_VULKAN_IMAGE_BUFFER_POOL (pool)
+        || GST_IS_VULKAN_BUFFER_POOL (pool);
   } else {
     GstVideoInfo vinfo;
 
     gst_video_info_init (&vinfo);
     gst_video_info_from_caps (&vinfo, caps);
-    size = vinfo.size;
+
+    size = GST_VIDEO_INFO_SIZE (&vinfo);
     min = max = 0;
     is_vulkan_pool = update_pool = FALSE;
   }
@@ -1374,12 +1401,12 @@ gst_vulkan_upload_decide_allocation (GstBaseTransform * bt, GstQuery * query)
   if (!pool || !is_vulkan_pool) {
     if (pool)
       gst_object_unref (pool);
-    pool = gst_vulkan_image_buffer_pool_new (vk_upload->device);
+    pool = gst_vulkan_upload_create_buffer_pool (vk_upload);
   }
 
   config = gst_buffer_pool_get_config (pool);
 
-  if (is_vulkan_pool) {
+  if (GST_IS_VULKAN_IMAGE_BUFFER_POOL (pool)) {
     gst_vulkan_image_buffer_pool_config_get_allocation_params (config, &usage,
         &mem_props, &layout, &access);
     /* these usage parameters are essential for upload */
@@ -1387,8 +1414,16 @@ gst_vulkan_upload_decide_allocation (GstBaseTransform * bt, GstQuery * query)
   }
 
   gst_buffer_pool_config_set_params (config, caps, size, min, max);
-  gst_vulkan_image_buffer_pool_config_set_allocation_params (config, usage,
-      mem_props, layout, access);
+
+  if (GST_IS_VULKAN_IMAGE_BUFFER_POOL (pool)) {
+    gst_vulkan_image_buffer_pool_config_set_allocation_params (config, usage,
+        mem_props, layout, access);
+  }
+
+  if (gst_buffer_pool_has_option (pool, GST_BUFFER_POOL_OPTION_VIDEO_META)) {
+    gst_buffer_pool_config_add_option (config,
+        GST_BUFFER_POOL_OPTION_VIDEO_META);
+  }
 
   if (!gst_buffer_pool_set_config (pool, config)) {
     GST_ERROR_OBJECT (pool, "Vulkan Image buffer pool doesn't support requested"
