@@ -63,6 +63,7 @@ enum
   PROP_FULLSCREEN,
   PROP_ROTATE_METHOD,
   PROP_DRM_DEVICE,
+  PROP_FORCE_ASPECT_RATIO,
   PROP_LAST
 };
 
@@ -159,7 +160,8 @@ gst_wayland_sink_class_init (GstWaylandSinkClass * klass)
   g_object_class_install_property (gobject_class, PROP_FULLSCREEN,
       g_param_spec_boolean ("fullscreen", "Fullscreen",
           "Whether the surface should be made fullscreen ", FALSE,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          G_PARAM_READWRITE | GST_PARAM_MUTABLE_PLAYING |
+          G_PARAM_STATIC_STRINGS));
 
   /**
    * waylandsink:rotate-method:
@@ -171,7 +173,8 @@ gst_wayland_sink_class_init (GstWaylandSinkClass * klass)
           "rotate method",
           "rotate method",
           GST_TYPE_VIDEO_ORIENTATION_METHOD, GST_VIDEO_ORIENTATION_IDENTITY,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          G_PARAM_READWRITE | GST_PARAM_MUTABLE_PLAYING |
+          G_PARAM_STATIC_STRINGS));
 
  /**
    * waylandsink:drm-device:
@@ -184,6 +187,18 @@ gst_wayland_sink_class_init (GstWaylandSinkClass * klass)
           NULL,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT_ONLY));
 
+ /**
+   * waylandsink:force-aspect-ratio:
+   *
+   * Since: 1.28
+   */
+  g_object_class_install_property (gobject_class, PROP_FORCE_ASPECT_RATIO,
+      g_param_spec_boolean ("force-aspect-ratio", "Force aspect ratio",
+          "When enabled, scaling will respect original aspect ratio",
+          TRUE,
+          G_PARAM_READWRITE | GST_PARAM_MUTABLE_PLAYING |
+          G_PARAM_STATIC_STRINGS));
+
 
  /**
   * waylandsink:render-rectangle:
@@ -201,18 +216,22 @@ gst_wayland_sink_init (GstWaylandSink * self)
 {
   g_mutex_init (&self->display_lock);
   g_mutex_init (&self->render_lock);
+  self->force_aspect_ratio = TRUE;
 }
 
+/* must be called with the OBJECT_LOCK */
 static void
 gst_wayland_sink_set_fullscreen (GstWaylandSink * self, gboolean fullscreen)
 {
   if (fullscreen == self->fullscreen)
     return;
 
-  g_mutex_lock (&self->render_lock);
   self->fullscreen = fullscreen;
-  gst_wl_window_ensure_fullscreen (self->window, fullscreen);
-  g_mutex_unlock (&self->render_lock);
+  if (self->window) {
+    g_mutex_lock (&self->render_lock);
+    gst_wl_window_ensure_fullscreen (self->window, fullscreen);
+    g_mutex_unlock (&self->render_lock);
+  }
 }
 
 static void
@@ -252,6 +271,23 @@ gst_wayland_sink_set_rotate_method (GstWaylandSink * self,
   GST_OBJECT_UNLOCK (self);
 }
 
+/* must be called with the OBJECT_LOCK */
+static void
+gst_wayland_sink_set_force_aspect_ratio (GstWaylandSink * self,
+    gboolean force_aspect_ratio)
+{
+  if (force_aspect_ratio == self->force_aspect_ratio)
+    return;
+
+  self->force_aspect_ratio = force_aspect_ratio;
+  if (self->window) {
+    g_mutex_lock (&self->render_lock);
+    gst_wl_window_set_force_aspect_ratio (self->window,
+        self->force_aspect_ratio);
+    g_mutex_unlock (&self->render_lock);
+  }
+}
+
 static void
 gst_wayland_sink_get_property (GObject * object,
     guint prop_id, GValue * value, GParamSpec * pspec)
@@ -279,6 +315,11 @@ gst_wayland_sink_get_property (GObject * object,
       g_value_set_string (value, self->drm_device);
       GST_OBJECT_UNLOCK (self);
       break;
+    case PROP_FORCE_ASPECT_RATIO:
+      GST_OBJECT_LOCK (self);
+      g_value_set_boolean (value, self->force_aspect_ratio);
+      GST_OBJECT_UNLOCK (self);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -294,6 +335,7 @@ gst_wayland_sink_set_property (GObject * object,
   switch (prop_id) {
     case PROP_DISPLAY:
       GST_OBJECT_LOCK (self);
+      g_free (self->display_name);
       self->display_name = g_value_dup_string (value);
       GST_OBJECT_UNLOCK (self);
       break;
@@ -307,10 +349,17 @@ gst_wayland_sink_set_property (GObject * object,
           FALSE);
       break;
     case PROP_DRM_DEVICE:
+      /* G_PARAM_CONSTRUCT_ONLY */
       GST_OBJECT_LOCK (self);
       self->drm_device = g_value_dup_string (value);
       GST_OBJECT_UNLOCK (self);
       break;
+    case PROP_FORCE_ASPECT_RATIO:
+      GST_OBJECT_LOCK (self);
+      gst_wayland_sink_set_force_aspect_ratio (self,
+          g_value_get_boolean (value));
+      GST_OBJECT_UNLOCK (self);
+      break;
     default:
       if (!gst_video_overlay_set_property (object, PROP_LAST, prop_id, value))
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -864,6 +913,8 @@ gst_wayland_sink_show_frame (GstVideoSink * vsink, GstBuffer * buffer)
           G_CALLBACK (on_window_closed), self, 0);
       gst_wl_window_set_rotate_method (self->window,
           self->current_rotate_method);
+      gst_wl_window_set_force_aspect_ratio (self->window,
+          self->force_aspect_ratio);
     }
   }
 
@@ -1132,6 +1183,8 @@ gst_wayland_sink_set_window_handle (GstVideoOverlay * overlay, guintptr handle)
             &self->render_lock);
         gst_wl_window_set_rotate_method (self->window,
             self->current_rotate_method);
+        gst_wl_window_set_force_aspect_ratio (self->window,
+            self->force_aspect_ratio);
       }
     } else {
       GST_ERROR_OBJECT (self, "Failed to find display handle, "
