@@ -71,11 +71,14 @@ _set_base_stats (GstStructure * s, GstWebRTCStatsType type, double ts,
 static GstStructure *
 _get_peer_connection_stats (GstWebRTCBin * webrtc)
 {
-  GstStructure *s = gst_structure_new_empty ("unused");
+  guint opened;
+  guint closed;
+  GstStructure *s = gst_structure_new_empty ("peer-connection-stats");
 
-  /* FIXME: datachannel */
-  gst_structure_set (s, "data-channels-opened", G_TYPE_UINT, 0,
-      "data-channels-closed", G_TYPE_UINT, 0, "data-channels-requested",
+  gst_webrtc_bin_get_peer_connection_stats (webrtc, &opened, &closed);
+
+  gst_structure_set (s, "data-channels-opened", G_TYPE_UINT, opened,
+      "data-channels-closed", G_TYPE_UINT, closed, "data-channels-requested",
       G_TYPE_UINT, 0, "data-channels-accepted", G_TYPE_UINT, 0, NULL);
 
   return s;
@@ -103,12 +106,12 @@ _gst_structure_take_structure (GstStructure * s, const char *fieldname,
 
 /* https://www.w3.org/TR/webrtc-stats/#remoteinboundrtpstats-dict* */
 static gboolean
-_get_stats_from_remote_rtp_source_stats (GstWebRTCBin * webrtc,
-    TransportStream * stream, const GstStructure * source_stats,
-    guint ssrc, guint clock_rate, const gchar * codec_id, const gchar * kind,
-    const gchar * transport_id, GstStructure * s)
+_get_stats_from_remote_rtp_source_stats (TransportStream * stream,
+    const GstStructure * source_stats, guint ssrc, guint clock_rate,
+    const gchar * codec_id, const gchar * kind, const gchar * transport_id,
+    GstStructure * s)
 {
-  gboolean have_rb = FALSE, internal = FALSE;
+  gboolean have_rb = FALSE;
   int lost;
   GstStructure *r_in;
   gchar *r_in_id, *out_id;
@@ -117,11 +120,10 @@ _get_stats_from_remote_rtp_source_stats (GstWebRTCBin * webrtc,
   double ts;
 
   gst_structure_get_double (s, "timestamp", &ts);
-  gst_structure_get (source_stats, "internal", G_TYPE_BOOLEAN, &internal,
-      "have-rb", G_TYPE_BOOLEAN, &have_rb, NULL);
+  gst_structure_get (source_stats, "have-rb", G_TYPE_BOOLEAN, &have_rb, NULL);
 
   /* This isn't what we're looking for */
-  if (internal == TRUE || have_rb == FALSE)
+  if (have_rb == FALSE)
     return FALSE;
 
   r_in_id = g_strdup_printf ("rtp-remote-inbound-stream-stats_%u", ssrc);
@@ -204,10 +206,9 @@ _get_stats_from_remote_rtp_source_stats (GstWebRTCBin * webrtc,
 /* https://www.w3.org/TR/webrtc-stats/#inboundrtpstats-dict*
    https://www.w3.org/TR/webrtc-stats/#outboundrtpstats-dict* */
 static void
-_get_stats_from_rtp_source_stats (GstWebRTCBin * webrtc,
-    TransportStream * stream, const GstStructure * source_stats,
-    const gchar * codec_id, const gchar * kind, const gchar * transport_id,
-    GstStructure * s)
+_get_stats_from_rtp_source_stats (TransportStream * stream,
+    const GstStructure * source_stats, const gchar * codec_id,
+    const gchar * kind, const gchar * transport_id, GstStructure * s)
 {
   guint ssrc, fir, pli, nack, jitter;
   int clock_rate;
@@ -336,12 +337,6 @@ _get_stats_from_rtp_source_stats (GstWebRTCBin * webrtc,
       }
     }
 
-    if (jb_stats)
-      gst_structure_get (jb_stats, "num-lost", G_TYPE_UINT64, &jb_lost,
-          "num-duplicates", G_TYPE_UINT64, &duplicates, "num-late",
-          G_TYPE_UINT64, &late, "rtx-success-count", G_TYPE_UINT64,
-          &rtx_success, NULL);
-
     in_id = g_strdup_printf ("rtp-inbound-stream-stats_%u", ssrc);
     r_out_id = g_strdup_printf ("rtp-remote-outbound-stream-stats_%u", ssrc);
 
@@ -357,20 +352,39 @@ _get_stats_from_rtp_source_stats (GstWebRTCBin * webrtc,
 
     /* RTCReceivedRtpStreamStats */
 
-    if (gst_structure_get_uint64 (source_stats, "packets-received", &packets))
+    if (gst_structure_get_uint64 (source_stats, "packets-received", &packets)) {
       gst_structure_set (in, "packets-received", G_TYPE_UINT64, packets, NULL);
+    } else {
+      gst_structure_set (in, "packets-received", G_TYPE_UINT64,
+          G_GUINT64_CONSTANT (0), NULL);
+    }
+
     if (jb_stats) {
+      gst_structure_get (jb_stats, "num-lost", G_TYPE_UINT64, &jb_lost,
+          "num-duplicates", G_TYPE_UINT64, &duplicates, "num-late",
+          G_TYPE_UINT64, &late, "rtx-success-count", G_TYPE_UINT64,
+          &rtx_success, NULL);
+
       gint64 packets_lost = jb_lost > G_MAXINT64 ?
           G_MAXINT64 : (gint64) jb_lost;
       gst_structure_set (in, "packets-lost", G_TYPE_INT64, packets_lost, NULL);
-    }
-    if (gst_structure_get_uint (source_stats, "jitter", &jitter))
-      gst_structure_set (in, "jitter", G_TYPE_DOUBLE,
-          CLOCK_RATE_VALUE_TO_SECONDS (jitter, clock_rate), NULL);
 
-    if (jb_stats)
       gst_structure_set (in, "packets-discarded", G_TYPE_UINT64, late,
           "packets-repaired", G_TYPE_UINT64, rtx_success, NULL);
+    } else {
+      gst_structure_set (in, "packets-lost", G_TYPE_INT64,
+          G_GINT64_CONSTANT (0), NULL);
+      gst_structure_set (in, "packets-discarded", G_TYPE_UINT64,
+          G_GUINT64_CONSTANT (0), "packets-repaired", G_TYPE_UINT64,
+          G_GUINT64_CONSTANT (0), NULL);
+    }
+
+    if (gst_structure_get_uint (source_stats, "jitter", &jitter)) {
+      gst_structure_set (in, "jitter", G_TYPE_DOUBLE,
+          CLOCK_RATE_VALUE_TO_SECONDS (jitter, clock_rate), NULL);
+    } else {
+      gst_structure_set (in, "jitter", G_TYPE_DOUBLE, 0.0, NULL);
+    }
 
     /*
        RTCReceivedRtpStreamStats
@@ -466,19 +480,20 @@ _get_stats_from_rtp_source_stats (GstWebRTCBin * webrtc,
     gst_structure_set (r_out, "codec-id", G_TYPE_STRING, codec_id, NULL);
     gst_structure_set (r_out, "transport-id", G_TYPE_STRING, transport_id,
         NULL);
+    if (kind)
+      gst_structure_set (r_out, "kind", G_TYPE_STRING, kind, NULL);
+
     /* XXX: mediaType, trackId */
 
     /* RTCSentRtpStreamStats */
 
+    guint sr_bytes = 0, sr_packets = 0;
     if (have_sr) {
-      guint sr_bytes, sr_packets;
-
-      if (gst_structure_get_uint (source_stats, "sr-octet-count", &sr_bytes))
-        gst_structure_set (r_out, "bytes-sent", G_TYPE_UINT, sr_bytes, NULL);
-      if (gst_structure_get_uint (source_stats, "sr-packet-count", &sr_packets))
-        gst_structure_set (r_out, "packets-sent", G_TYPE_UINT, sr_packets,
-            NULL);
+      gst_structure_get_uint (source_stats, "sr-octet-count", &sr_bytes);
+      gst_structure_get_uint (source_stats, "sr-packet-count", &sr_packets);
     }
+    gst_structure_set (r_out, "bytes-sent", G_TYPE_UINT, sr_bytes, NULL);
+    gst_structure_set (r_out, "packets-sent", G_TYPE_UINT, sr_packets, NULL);
 
     /* RTCSentRtpStreamStats:
 
@@ -581,9 +596,9 @@ _get_stats_from_ice_candidates (GstWebRTCBin * webrtc,
       can->stream_id, can->ipaddr, can->port);
   stats = gst_structure_new_empty (id);
 
-  if (strcmp (candidate_tag, "local")) {
+  if (g_str_equal (candidate_tag, "local")) {
     type = GST_WEBRTC_STATS_LOCAL_CANDIDATE;
-  } else if (strcmp (candidate_tag, "remote")) {
+  } else if (g_str_equal (candidate_tag, "remote")) {
     type = GST_WEBRTC_STATS_REMOTE_CANDIDATE;
   } else {
     GST_WARNING_OBJECT (webrtc, "Invalid ice candidate tag: %s", candidate_tag);
@@ -600,6 +615,11 @@ _get_stats_from_ice_candidates (GstWebRTCBin * webrtc,
      long                priority;
      DOMString           url;
      DOMString           relayProtocol;
+     DOMString           foundation;
+     DOMString           relatedAddress;
+     long                relatedPort;
+     DOMString           usernameFragment;
+     RTCIceTcpCandidateType tcpType;
    */
 
   if (transport_id)
@@ -615,6 +635,21 @@ _get_stats_from_ice_candidates (GstWebRTCBin * webrtc,
         NULL);
   if (can->url)
     gst_structure_set (stats, "url", G_TYPE_STRING, can->url, NULL);
+  if (can->ABI.abi.foundation)
+    gst_structure_set (stats, "foundation", G_TYPE_STRING,
+        can->ABI.abi.foundation, NULL);
+  if (can->ABI.abi.related_address)
+    gst_structure_set (stats, "related-address", G_TYPE_STRING,
+        can->ABI.abi.related_address, NULL);
+  if (can->ABI.abi.related_port)
+    gst_structure_set (stats, "related-port", G_TYPE_UINT,
+        can->ABI.abi.related_port, NULL);
+  if (can->ABI.abi.username_fragment)
+    gst_structure_set (stats, "username-fragment", G_TYPE_STRING,
+        can->ABI.abi.username_fragment, NULL);
+  if (can->ABI.abi.tcp_type != GST_WEBRTC_ICE_TCP_CANDIDATE_TYPE_NONE)
+    gst_structure_set (stats, "tcp-type",
+        GST_TYPE_WEBRTC_ICE_TCP_CANDIDATE_TYPE, can->ABI.abi.tcp_type, NULL);
 
   gst_structure_set (s, id, GST_TYPE_STRUCTURE, stats, NULL);
   gst_structure_free (stats);
@@ -792,13 +827,16 @@ _get_codec_stats_from_pad (GstWebRTCBin * webrtc, GstPad * pad,
 
   if (wpad->received_caps)
     caps = gst_caps_ref (wpad->received_caps);
+  else
+    caps = gst_pad_get_current_caps (pad);
+
   GST_DEBUG_OBJECT (pad, "Pad caps are: %" GST_PTR_FORMAT, caps);
   if (caps && gst_caps_is_fixed (caps)) {
     GstStructure *caps_s = gst_caps_get_structure (caps, 0);
     gint pt;
     const gchar *encoding_name, *media, *encoding_params;
-    GstSDPMedia sdp_media = { 0 };
     guint channels = 0;
+    const gchar *fmtp;
 
     if (gst_structure_get_int (caps_s, "payload", &pt))
       gst_structure_set (stats, "payload-type", G_TYPE_UINT, pt, NULL);
@@ -834,15 +872,10 @@ _get_codec_stats_from_pad (GstWebRTCBin * webrtc, GstPad * pad,
     else
       gst_structure_set (stats, "codec-type", G_TYPE_STRING, "encode", NULL);
 
-    gst_sdp_media_init (&sdp_media);
-    if (gst_sdp_media_set_media_from_caps (caps, &sdp_media) == GST_SDP_OK) {
-      const gchar *fmtp = gst_sdp_media_get_attribute_val (&sdp_media, "fmtp");
-
-      if (fmtp) {
-        gst_structure_set (stats, "sdp-fmtp-line", G_TYPE_STRING, fmtp, NULL);
-      }
+    fmtp = gst_structure_get_string (caps_s, "a-fmtp");
+    if (fmtp) {
+      gst_structure_set (stats, "sdp-fmtp-line", G_TYPE_STRING, fmtp, NULL);
     }
-    gst_sdp_media_uninit (&sdp_media);
 
     /* FIXME: transportId */
   }
@@ -869,45 +902,55 @@ _get_codec_stats_from_pad (GstWebRTCBin * webrtc, GstPad * pad,
 
 struct transport_stream_stats
 {
-  GstWebRTCBin *webrtc;
   TransportStream *stream;
   char *transport_id;
   char *codec_id;
+  guint ssrc;
   const char *kind;
   guint clock_rate;
   GValueArray *source_stats;
-  GstStructure *s;
+
+  GstStructure *s;              /* Return value stats accumulator */
 };
 
 static gboolean
-webrtc_stats_get_from_transport (SsrcMapItem * entry,
+webrtc_stats_get_from_transport_for_one_ssrc (SsrcMapItem * entry,
     struct transport_stream_stats *ts_stats)
 {
   double ts;
   int i;
 
+  /* We're only interested in the map entry for the ssrc for the
+   * pad under inspection */
+  if (ts_stats->ssrc != entry->ssrc)
+    return FALSE;               /* Continue iterating */
+
   gst_structure_get_double (ts_stats->s, "timestamp", &ts);
 
   /* construct stats objects */
   for (i = 0; i < ts_stats->source_stats->n_values; i++) {
-    const GstStructure *stats;
     const GValue *val = g_value_array_get_nth (ts_stats->source_stats, i);
+    const GstStructure *stats = gst_value_get_structure (val);
+
     guint stats_ssrc = 0;
 
-    stats = gst_value_get_structure (val);
-
-    /* skip foreign sources */
     if (gst_structure_get_uint (stats, "ssrc", &stats_ssrc) &&
-        entry->ssrc == stats_ssrc)
-      _get_stats_from_rtp_source_stats (ts_stats->webrtc, ts_stats->stream,
-          stats, ts_stats->codec_id, ts_stats->kind, ts_stats->transport_id,
-          ts_stats->s);
-    else if (gst_structure_get_uint (stats, "rb-ssrc", &stats_ssrc)
-        && entry->ssrc == stats_ssrc)
-      _get_stats_from_remote_rtp_source_stats (ts_stats->webrtc,
-          ts_stats->stream, stats, entry->ssrc, ts_stats->clock_rate,
+        entry->ssrc == stats_ssrc) {
+      GST_TRACE ("Found source stats for ssrc %u: %" GST_PTR_FORMAT, stats_ssrc,
+          stats);
+      _get_stats_from_rtp_source_stats (ts_stats->stream, stats,
           ts_stats->codec_id, ts_stats->kind, ts_stats->transport_id,
           ts_stats->s);
+    }
+
+    if (gst_structure_get_uint (stats, "rb-ssrc", &stats_ssrc)
+        && entry->ssrc == stats_ssrc) {
+      GST_TRACE ("Found remote source stats for ssrc %u: %" GST_PTR_FORMAT,
+          stats_ssrc, stats);
+      _get_stats_from_remote_rtp_source_stats (ts_stats->stream, stats,
+          entry->ssrc, ts_stats->clock_rate, ts_stats->codec_id, ts_stats->kind,
+          ts_stats->transport_id, ts_stats->s);
+    }
   }
 
   /* we want to look at all the entries */
@@ -919,14 +962,14 @@ _get_stats_from_pad (GstWebRTCBin * webrtc, GstPad * pad, GstStructure * s)
 {
   GstWebRTCBinPad *wpad = GST_WEBRTC_BIN_PAD (pad);
   struct transport_stream_stats ts_stats = { NULL, };
-  guint ssrc, clock_rate;
+  guint clock_rate;
   GObject *rtp_session;
   GObject *gst_rtp_session;
   GstStructure *rtp_stats, *twcc_stats;
   GstWebRTCKind kind;
 
-  _get_codec_stats_from_pad (webrtc, pad, s, &ts_stats.codec_id, &ssrc,
-      &clock_rate);
+  _get_codec_stats_from_pad (webrtc, pad, s, &ts_stats.codec_id,
+      &ts_stats.ssrc, &clock_rate);
 
   if (!wpad->trans)
     goto out;
@@ -974,9 +1017,10 @@ _get_stats_from_pad (GstWebRTCBin * webrtc, GstPad * pad, GstStructure * s)
       ts_stats.source_stats->n_values, ts_stats.stream->transport);
 
   ts_stats.s = s;
+  ts_stats.clock_rate = clock_rate;
 
   transport_stream_find_ssrc_map_item (ts_stats.stream, &ts_stats,
-      (FindSsrcMapFunc) webrtc_stats_get_from_transport);
+      (FindSsrcMapFunc) webrtc_stats_get_from_transport_for_one_ssrc);
 
   g_clear_object (&rtp_session);
   g_clear_object (&gst_rtp_session);
@@ -991,6 +1035,28 @@ out:
   return TRUE;
 }
 
+static void
+_get_data_channel_transport_stats (GstWebRTCBin * webrtc, GstStructure * s)
+{
+  struct transport_stream_stats ts_stats = {
+    NULL,
+  };
+  GObject *gst_rtp_session;
+
+  if (!webrtc->priv->data_channel_transport)
+    return;
+
+  ts_stats.stream = webrtc->priv->data_channel_transport;
+
+  g_signal_emit_by_name (webrtc->rtpbin, "get-session",
+      ts_stats.stream->session_id, &gst_rtp_session);
+
+  ts_stats.transport_id =
+      _get_stats_from_dtls_transport (webrtc, ts_stats.stream->transport,
+      GST_WEBRTC_ICE_STREAM (ts_stats.stream->stream), NULL, s);
+  g_clear_pointer (&ts_stats.transport_id, g_free);
+}
+
 GstStructure *
 gst_webrtc_bin_create_stats (GstWebRTCBin * webrtc, GstPad * pad)
 {
@@ -1015,6 +1081,8 @@ gst_webrtc_bin_create_stats (GstWebRTCBin * webrtc, GstPad * pad)
     gst_structure_free (pc_stats);
   }
 
+  _get_data_channel_transport_stats (webrtc, s);
+
   if (pad)
     _get_stats_from_pad (webrtc, pad, s);
   else
