@@ -23,22 +23,19 @@
 
 #include "transportsendbin.h"
 #include "utils.h"
+#include "gst/webrtc/webrtc-priv.h"
 
 /*
- *           ,------------------------transport_send_%u-------------------------,
- *           ;                          ,-----dtlssrtpenc---,                   ;
- * data_sink o--------------------------o data_sink         ;                   ;
- *           ;                          ;                   ;  ,---nicesink---, ;
- *  rtp_sink o--------------------------o rtp_sink_0    src o--o sink         ; ;
- *           ;                          ;                   ;  '--------------' ;
- *           ;   ,--outputselector--, ,-o rtcp_sink_0       ;                   ;
- *           ;   ;            src_0 o-' '-------------------'                   ;
- * rtcp_sink ;---o sink             ;   ,----dtlssrtpenc----,  ,---nicesink---, ;
- *           ;   ;            src_1 o---o rtcp_sink_0   src o--o sink         ; ;
- *           ;   '------------------'   '-------------------'  '--------------' ;
- *           '------------------------------------------------------------------'
+ *           ,--------------transport_send_%u-------- ---,
+ *           ;   ,-----dtlssrtpenc---,                   ;
+ * data_sink o---o data_sink         ;                   ;
+ *           ;   ;                   ;  ,---nicesink---, ;
+ *  rtp_sink o---o rtp_sink_0    src o--o sink         ; ;
+ *           ;   ;                   ;  '--------------' ;
+ * rtcp_sink o---o rtcp_sink_0       ;                   ;
+ *           ;   '-------------------'
+ *           '-------------------------------------------'
  *
- * outputselecter is used to switch between rtcp-mux and no rtcp-mux
  *
  * FIXME: Do we need a valve drop=TRUE for the no RTCP case?
  */
@@ -73,7 +70,6 @@ enum
 {
   PROP_0,
   PROP_STREAM,
-  PROP_RTCP_MUX,
 };
 
 #define TSB_GET_LOCK(tsb) (&tsb->lock)
@@ -82,24 +78,6 @@ enum
 
 static void cleanup_blocks (TransportSendBin * send);
 
-static void
-_set_rtcp_mux (TransportSendBin * send, gboolean rtcp_mux)
-{
-  GstPad *active_pad;
-
-  if (rtcp_mux)
-    active_pad = gst_element_get_static_pad (send->outputselector, "src_0");
-  else
-    active_pad = gst_element_get_static_pad (send->outputselector, "src_1");
-  send->rtcp_mux = rtcp_mux;
-  GST_OBJECT_UNLOCK (send);
-
-  g_object_set (send->outputselector, "active-pad", active_pad, NULL);
-
-  gst_object_unref (active_pad);
-  GST_OBJECT_LOCK (send);
-}
-
 static void
 transport_send_bin_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -112,9 +90,6 @@ transport_send_bin_set_property (GObject * object, guint prop_id,
       /* XXX: weak-ref this? Note, it's construct-only so can't be changed later */
       send->stream = TRANSPORT_STREAM (g_value_get_object (value));
       break;
-    case PROP_RTCP_MUX:
-      _set_rtcp_mux (send, g_value_get_boolean (value));
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -133,9 +108,6 @@ transport_send_bin_get_property (GObject * object, guint prop_id,
     case PROP_STREAM:
       g_value_set_object (value, send->stream);
       break;
-    case PROP_RTCP_MUX:
-      g_value_set_boolean (value, send->rtcp_mux);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -200,9 +172,9 @@ transport_send_bin_change_state (GstElement * element,
        * arguably the element should be able to deal with this itself or
        * we should only add it once/if we get the encoding keys */
       TSB_LOCK (send);
-      gst_element_set_locked_state (send->rtp_ctx.dtlssrtpenc, TRUE);
-      gst_element_set_locked_state (send->rtcp_ctx.dtlssrtpenc, TRUE);
+      gst_element_set_locked_state (send->dtlssrtpenc, TRUE);
       send->active = TRUE;
+      send->has_clientness = FALSE;
       TSB_UNLOCK (send);
       break;
     }
@@ -213,20 +185,12 @@ transport_send_bin_change_state (GstElement * element,
       /* RTP */
       /* unblock the encoder once the key is set, this should also be automatic */
       elem = send->stream->transport->dtlssrtpenc;
-      send->rtp_ctx.rtp_block = block_peer_pad (elem, "rtp_sink_0");
+      send->rtp_block = block_peer_pad (elem, "rtp_sink_0");
       /* Also block the RTCP pad on the RTP encoder, in case we mux RTCP */
-      send->rtp_ctx.rtcp_block = block_peer_pad (elem, "rtcp_sink_0");
+      send->rtcp_block = block_peer_pad (elem, "rtcp_sink_0");
       /* unblock ice sink once a connection is made, this should also be automatic */
       elem = send->stream->transport->transport->sink;
-      send->rtp_ctx.nice_block = block_peer_pad (elem, "sink");
 
-      /* RTCP */
-      elem = send->stream->rtcp_transport->dtlssrtpenc;
-      /* Block the RTCP DTLS encoder */
-      send->rtcp_ctx.rtcp_block = block_peer_pad (elem, "rtcp_sink_0");
-      /* unblock ice sink once a connection is made, this should also be automatic */
-      elem = send->stream->rtcp_transport->transport->sink;
-      send->rtcp_ctx.nice_block = block_peer_pad (elem, "sink");
       TSB_UNLOCK (send);
       break;
     }
@@ -256,8 +220,7 @@ transport_send_bin_change_state (GstElement * element,
       send->active = FALSE;
       cleanup_blocks (send);
 
-      gst_element_set_locked_state (send->rtp_ctx.dtlssrtpenc, FALSE);
-      gst_element_set_locked_state (send->rtcp_ctx.dtlssrtpenc, FALSE);
+      gst_element_set_locked_state (send->dtlssrtpenc, FALSE);
       TSB_UNLOCK (send);
 
       break;
@@ -272,13 +235,7 @@ transport_send_bin_change_state (GstElement * element,
 static void
 _on_dtls_enc_key_set (GstElement * dtlssrtpenc, TransportSendBin * send)
 {
-  TransportSendBinDTLSContext *ctx;
-
-  if (dtlssrtpenc == send->rtp_ctx.dtlssrtpenc)
-    ctx = &send->rtp_ctx;
-  else if (dtlssrtpenc == send->rtcp_ctx.dtlssrtpenc)
-    ctx = &send->rtcp_ctx;
-  else {
+  if (dtlssrtpenc != send->dtlssrtpenc) {
     GST_WARNING_OBJECT (send,
         "Received dtls-enc key info for unknown element %" GST_PTR_FORMAT,
         dtlssrtpenc);
@@ -293,24 +250,40 @@ _on_dtls_enc_key_set (GstElement * dtlssrtpenc, TransportSendBin * send)
   }
 
   GST_LOG_OBJECT (send, "Unblocking %" GST_PTR_FORMAT " pads", dtlssrtpenc);
-  _free_pad_block (ctx->rtp_block);
-  _free_pad_block (ctx->rtcp_block);
-  ctx->rtp_block = ctx->rtcp_block = NULL;
+  _free_pad_block (send->rtp_block);
+  _free_pad_block (send->rtcp_block);
+  send->rtp_block = send->rtcp_block = NULL;
 
 done:
   TSB_UNLOCK (send);
 }
 
+static void
+maybe_start_enc (TransportSendBin * send)
+{
+  GstWebRTCICEConnectionState state;
+
+  if (!send->has_clientness) {
+    GST_LOG_OBJECT (send, "Can't start DTLS because doesn't know client-ness");
+    return;
+  }
+
+  g_object_get (send->stream->transport->transport, "state", &state, NULL);
+  if (state != GST_WEBRTC_ICE_CONNECTION_STATE_CONNECTED &&
+      state != GST_WEBRTC_ICE_CONNECTION_STATE_COMPLETED) {
+    GST_LOG_OBJECT (send, "Can't start DTLS yet because ICE is not connected.");
+    return;
+  }
+
+  gst_element_set_locked_state (send->dtlssrtpenc, FALSE);
+  gst_element_sync_state_with_parent (send->dtlssrtpenc);
+}
+
 static void
 _on_notify_dtls_client_status (GstElement * dtlssrtpenc,
     GParamSpec * pspec, TransportSendBin * send)
 {
-  TransportSendBinDTLSContext *ctx;
-  if (dtlssrtpenc == send->rtp_ctx.dtlssrtpenc)
-    ctx = &send->rtp_ctx;
-  else if (dtlssrtpenc == send->rtcp_ctx.dtlssrtpenc)
-    ctx = &send->rtcp_ctx;
-  else {
+  if (dtlssrtpenc != send->dtlssrtpenc) {
     GST_WARNING_OBJECT (send,
         "Received dtls-enc client mode for unknown element %" GST_PTR_FORMAT,
         dtlssrtpenc);
@@ -324,11 +297,12 @@ _on_notify_dtls_client_status (GstElement * dtlssrtpenc,
     goto done;
   }
 
+  send->has_clientness = TRUE;
   GST_DEBUG_OBJECT (send,
-      "DTLS-SRTP encoder configured. Unlocking it and changing state %"
-      GST_PTR_FORMAT, ctx->dtlssrtpenc);
-  gst_element_set_locked_state (ctx->dtlssrtpenc, FALSE);
-  gst_element_sync_state_with_parent (ctx->dtlssrtpenc);
+      "DTLS-SRTP encoder configured. Unlocking it and maybe changing state %"
+      GST_PTR_FORMAT, dtlssrtpenc);
+  maybe_start_enc (send);
+
 done:
   TSB_UNLOCK (send);
 }
@@ -337,116 +311,62 @@ static void
 _on_notify_ice_connection_state (GstWebRTCICETransport * transport,
     GParamSpec * pspec, TransportSendBin * send)
 {
-  GstWebRTCICEConnectionState state;
-
-  g_object_get (transport, "state", &state, NULL);
-
-  if (state == GST_WEBRTC_ICE_CONNECTION_STATE_CONNECTED ||
-      state == GST_WEBRTC_ICE_CONNECTION_STATE_COMPLETED) {
-    TSB_LOCK (send);
-    if (transport == send->stream->transport->transport) {
-      if (send->rtp_ctx.nice_block) {
-        GST_LOG_OBJECT (send, "Unblocking pad %" GST_PTR_FORMAT,
-            send->rtp_ctx.nice_block->pad);
-        _free_pad_block (send->rtp_ctx.nice_block);
-        send->rtp_ctx.nice_block = NULL;
-      }
-    } else if (transport == send->stream->rtcp_transport->transport) {
-      if (send->rtcp_ctx.nice_block) {
-        GST_LOG_OBJECT (send, "Unblocking pad %" GST_PTR_FORMAT,
-            send->rtcp_ctx.nice_block->pad);
-        _free_pad_block (send->rtcp_ctx.nice_block);
-        send->rtcp_ctx.nice_block = NULL;
-      }
-    }
-    TSB_UNLOCK (send);
-  }
-}
-
-static void
-tsb_setup_ctx (TransportSendBin * send, TransportSendBinDTLSContext * ctx,
-    GstWebRTCDTLSTransport * transport)
-{
-  GstElement *dtlssrtpenc, *nicesink;
-
-  dtlssrtpenc = ctx->dtlssrtpenc = transport->dtlssrtpenc;
-  nicesink = ctx->nicesink = transport->transport->sink;
-
-  /* unblock the encoder once the key is set */
-  g_signal_connect (dtlssrtpenc, "on-key-set",
-      G_CALLBACK (_on_dtls_enc_key_set), send);
-  /* Bring the encoder up to current state only once the is-client prop is set */
-  g_signal_connect (dtlssrtpenc, "notify::is-client",
-      G_CALLBACK (_on_notify_dtls_client_status), send);
-  gst_bin_add (GST_BIN (send), GST_ELEMENT (dtlssrtpenc));
-
-  /* unblock ice sink once it signals a connection */
-  g_signal_connect (transport->transport, "notify::state",
-      G_CALLBACK (_on_notify_ice_connection_state), send);
-  gst_bin_add (GST_BIN (send), GST_ELEMENT (nicesink));
-
-  if (!gst_element_link_pads (GST_ELEMENT (dtlssrtpenc), "src", nicesink,
-          "sink"))
-    g_warn_if_reached ();
+  TSB_LOCK (send);
+  maybe_start_enc (send);
+  TSB_UNLOCK (send);
 }
 
 static void
 transport_send_bin_constructed (GObject * object)
 {
   TransportSendBin *send = TRANSPORT_SEND_BIN (object);
-  GstWebRTCDTLSTransport *transport;
   GstPadTemplate *templ;
   GstPad *ghost, *pad;
 
   g_return_if_fail (send->stream);
 
-  g_object_bind_property (send, "rtcp-mux", send->stream, "rtcp-mux",
-      G_BINDING_BIDIRECTIONAL);
+  send->dtlssrtpenc = send->stream->transport->dtlssrtpenc;
+  send->nicesink = send->stream->transport->transport->sink;
 
-  /* Output selector to direct the RTCP for muxed-mode */
-  send->outputselector = gst_element_factory_make ("output-selector", NULL);
-  gst_bin_add (GST_BIN (send), send->outputselector);
+  /* unblock the encoder once the key is set */
+  g_signal_connect (send->dtlssrtpenc, "on-key-set",
+      G_CALLBACK (_on_dtls_enc_key_set), send);
+  /* Bring the encoder up to current state only once the is-client prop is set */
+  g_signal_connect (send->dtlssrtpenc, "notify::is-client",
+      G_CALLBACK (_on_notify_dtls_client_status), send);
+  /* unblock ice sink once it signals a connection */
+  g_signal_connect (send->stream->transport->transport, "notify::state",
+      G_CALLBACK (_on_notify_ice_connection_state), send);
 
-  /* RTP */
-  transport = send->stream->transport;
-  /* Do the common init for the context struct */
-  tsb_setup_ctx (send, &send->rtp_ctx, transport);
+  gst_bin_add (GST_BIN (send), GST_ELEMENT (send->dtlssrtpenc));
+  gst_bin_add (GST_BIN (send), GST_ELEMENT (send->nicesink));
 
-  templ = _find_pad_template (transport->dtlssrtpenc,
-      GST_PAD_SINK, GST_PAD_REQUEST, "rtp_sink_%d");
-  pad = gst_element_request_pad (transport->dtlssrtpenc, templ, "rtp_sink_0",
-      NULL);
-
-  if (!gst_element_link_pads (GST_ELEMENT (send->outputselector), "src_0",
-          GST_ELEMENT (transport->dtlssrtpenc), "rtcp_sink_0"))
+  if (!gst_element_link_pads (GST_ELEMENT (send->dtlssrtpenc), "src",
+          send->nicesink, "sink"))
     g_warn_if_reached ();
 
+  templ = _find_pad_template (send->dtlssrtpenc, GST_PAD_SINK, GST_PAD_REQUEST,
+      "rtp_sink_%d");
+  pad = gst_element_request_pad (send->dtlssrtpenc, templ, "rtp_sink_0", NULL);
+
   ghost = gst_ghost_pad_new ("rtp_sink", pad);
   gst_element_add_pad (GST_ELEMENT (send), ghost);
   gst_object_unref (pad);
 
   /* push the data stream onto the RTP dtls element */
-  templ = _find_pad_template (transport->dtlssrtpenc,
-      GST_PAD_SINK, GST_PAD_REQUEST, "data_sink");
-  pad = gst_element_request_pad (transport->dtlssrtpenc, templ, "data_sink",
-      NULL);
+  templ = _find_pad_template (send->dtlssrtpenc, GST_PAD_SINK, GST_PAD_REQUEST,
+      "data_sink");
+  pad = gst_element_request_pad (send->dtlssrtpenc, templ, "data_sink", NULL);
 
   ghost = gst_ghost_pad_new ("data_sink", pad);
   gst_element_add_pad (GST_ELEMENT (send), ghost);
   gst_object_unref (pad);
 
   /* RTCP */
-  transport = send->stream->rtcp_transport;
   /* Do the common init for the context struct */
-  tsb_setup_ctx (send, &send->rtcp_ctx, transport);
-  templ = _find_pad_template (transport->dtlssrtpenc,
-      GST_PAD_SINK, GST_PAD_REQUEST, "rtcp_sink_%d");
-
-  if (!gst_element_link_pads (GST_ELEMENT (send->outputselector), "src_1",
-          GST_ELEMENT (transport->dtlssrtpenc), "rtcp_sink_0"))
-    g_warn_if_reached ();
-
-  pad = gst_element_get_static_pad (send->outputselector, "sink");
+  templ = _find_pad_template (send->dtlssrtpenc, GST_PAD_SINK, GST_PAD_REQUEST,
+      "rtcp_sink_%d");
+  pad = gst_element_request_pad (send->dtlssrtpenc, templ, "rtcp_sink_0", NULL);
 
   ghost = gst_ghost_pad_new ("rtcp_sink", pad);
   gst_element_add_pad (GST_ELEMENT (send), ghost);
@@ -455,30 +375,18 @@ transport_send_bin_constructed (GObject * object)
   G_OBJECT_CLASS (parent_class)->constructed (object);
 }
 
-static void
-cleanup_ctx_blocks (TransportSendBinDTLSContext * ctx)
-{
-  if (ctx->rtp_block) {
-    _free_pad_block (ctx->rtp_block);
-    ctx->rtp_block = NULL;
-  }
-
-  if (ctx->rtcp_block) {
-    _free_pad_block (ctx->rtcp_block);
-    ctx->rtcp_block = NULL;
-  }
-
-  if (ctx->nice_block) {
-    _free_pad_block (ctx->nice_block);
-    ctx->nice_block = NULL;
-  }
-}
-
 static void
 cleanup_blocks (TransportSendBin * send)
 {
-  cleanup_ctx_blocks (&send->rtp_ctx);
-  cleanup_ctx_blocks (&send->rtcp_ctx);
+  if (send->rtp_block) {
+    _free_pad_block (send->rtp_block);
+    send->rtp_block = NULL;
+  }
+
+  if (send->rtcp_block) {
+    _free_pad_block (send->rtcp_block);
+    send->rtcp_block = NULL;
+  }
 }
 
 static void
@@ -487,14 +395,11 @@ transport_send_bin_dispose (GObject * object)
   TransportSendBin *send = TRANSPORT_SEND_BIN (object);
 
   TSB_LOCK (send);
-  if (send->rtp_ctx.nicesink) {
-    g_signal_handlers_disconnect_by_data (send->rtp_ctx.nicesink, send);
-    send->rtp_ctx.nicesink = NULL;
-  }
-  if (send->rtcp_ctx.nicesink) {
-    g_signal_handlers_disconnect_by_data (send->rtcp_ctx.nicesink, send);
-    send->rtcp_ctx.nicesink = NULL;
+  if (send->nicesink) {
+    g_signal_handlers_disconnect_by_data (send->nicesink, send);
+    send->nicesink = NULL;
   }
+
   cleanup_blocks (send);
 
   TSB_UNLOCK (send);
@@ -604,8 +509,9 @@ transport_send_bin_class_init (TransportSendBinClass * klass)
   gst_element_class_add_static_pad_template (element_class,
       &data_sink_template);
 
-  gst_element_class_set_metadata (element_class, "WebRTC Transport Send Bin",
-      "Filter/Network/WebRTC", "A bin for webrtc connections",
+  gst_element_class_set_static_metadata (element_class,
+      "WebRTC Transport Send Bin", "Filter/Network/WebRTC",
+      "A bin for webrtc connections",
       "Matthew Waters <matthew@centricular.com>");
 
   gobject_class->constructed = transport_send_bin_constructed;
@@ -623,12 +529,6 @@ transport_send_bin_class_init (TransportSendBinClass * klass)
           "The TransportStream for this sending bin",
           transport_stream_get_type (),
           G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
-
-  g_object_class_install_property (gobject_class,
-      PROP_RTCP_MUX,
-      g_param_spec_boolean ("rtcp-mux", "RTCP Mux",
-          "Whether RTCP packets are muxed with RTP packets",
-          FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
