@@ -522,7 +522,8 @@ gst_x265_enc_class_init (GstX265EncClass * klass)
   g_object_class_install_property (gobject_class, PROP_KEY_INT_MAX,
       g_param_spec_int ("key-int-max", "Max key frame",
           "Maximal distance between two key-frames (0 = x265 default / 250)",
-          0, G_MAXINT32, PROP_KEY_INT_MAX_DEFAULT, G_PARAM_READWRITE));
+          0, G_MAXINT32, PROP_KEY_INT_MAX_DEFAULT,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gst_element_class_set_static_metadata (element_class,
       "x265enc", "Codec/Encoder/Video", "H265 Encoder",
@@ -1247,7 +1248,9 @@ gst_x265_enc_set_src_caps (GstX265Enc * encoder, GstCaps * caps)
 
   tags = gst_tag_list_new_empty ();
   gst_tag_list_add (tags, GST_TAG_MERGE_REPLACE, GST_TAG_ENCODER, "x265",
-      GST_TAG_ENCODER_VERSION, x265_version_str, NULL);
+      GST_TAG_ENCODER_VERSION, x265_version_str,
+      GST_TAG_MAXIMUM_BITRATE, encoder->bitrate * 1024,
+      GST_TAG_NOMINAL_BITRATE, encoder->bitrate * 1024, NULL);
   gst_video_encoder_merge_tags (GST_VIDEO_ENCODER (encoder), tags,
       GST_TAG_MERGE_REPLACE);
   gst_tag_list_unref (tags);
