@@ -51,6 +51,7 @@
 #include "gstd3d12swapchainsink.h"
 #include "gstd3d12mipmapping.h"
 #include "gstd3d12deinterlace.h"
+#include "gstd3d12remap.h"
 #include <windows.h>
 #include <versionhelpers.h>
 #include <wrl.h>
@@ -66,6 +67,10 @@
 #include "gstd3d12upload.h"
 #endif
 
+#ifdef HAVE_WGC
+#include "gstd3d12graphicscapture.h"
+#endif
+
 /* *INDENT-OFF* */
 using namespace Microsoft::WRL;
 /* *INDENT-ON* */
@@ -73,6 +78,9 @@ using namespace Microsoft::WRL;
 static void
 plugin_deinit (gpointer data)
 {
+#ifdef HAVE_WGC
+  gst_d3d12_graphics_capture_deinit ();
+#endif
   gst_d3d12_ipc_client_deinit ();
   gst_d3d12_flush_all_devices ();
 }
@@ -192,6 +200,8 @@ plugin_init (GstPlugin * plugin)
       "d3d12mipmapping", GST_RANK_NONE, GST_TYPE_D3D12_MIP_MAPPING);
   gst_element_register (plugin,
       "d3d12deinterlace", GST_RANK_NONE, GST_TYPE_D3D12_DEINTERLACE);
+  gst_element_register (plugin,
+      "d3d12remap", GST_RANK_NONE, GST_TYPE_D3D12_REMAP);
 
   g_object_set_data_full (G_OBJECT (plugin),
       "plugin-d3d12-shutdown", (gpointer) "shutdown-data",
